/*
// EF_Image.h
//
// $Id: EF_Image.h,v 1.2 1998/02/08 06:08:52 mmp Exp $
*/


#ifndef EF_IMAGE_H
#define EF_IMAGE_H


#ifdef __cplusplus
extern "C" {
#endif

#define FOUR_BYTES long

  /* This defines the EF_Image type. This is the main type you */
  /* will need to work with. You plugin gets one or more of    */
  /* these, modifies them, and returns. The pixels are stored  */
  /* as a continuous block of memory. Row 1, then Row 2, etc.. */
  /* Each pixel is four bytes. With color encoded as 8bits per */
  /* channel. RGBA.                                            */
  typedef struct _EF_Image
  {

    FOUR_BYTES *pixels;
    short       xSize;
    short       ySize;
  
  } EF_Image;

  /* This macro provides an easy way to get at pixels in an    */
  /* image. You can of course calculate the offset yourself if */
  /* you prefer. This macro can be used as either an L-value   */
  /* or an R-value. i.e. both of the following are valid.      */
  /* EF_Image(image,x,y) = color;                              */
  /* color = EF_Image(image,x,y);                              */
#define EF_ImageXY(image,x,y) ((image)->pixels[(y)*((image)->xSize)+(x)])

  /* This is just a safe version of the routine above. It uses */
  /* _mod_ to make sure we don't try to read or write outside  */
  /* image bounds. i.e. if your image has 240 rows, and you    */
  /* try to write image location (100,245) with the macro above*/
  /* a segmentation fault is likely. With this macro you are   */
  /* saved. You will access location (100,5), which may or may */
  /* not be what you intended.                                 */
#define EF_ImageSafeXY(image,x,y) ((image)->pixels[(((y)+4*(image)->ySize)%(image)->ySize)*((image)->xSize)+(((x)+4*(image)->xSize)%(image)->xSize)])

  /* These macros give you an easy way to access the color     */
  /* channels in a four byte pixel. For example                */
  /* long color;                                               */
  /* color = EF_Image (image, x, y);                           */
  /* RED(color) = GREEN(color);                                */
  /* EF_Image (image, x, y) = color;                           */
#define RED(pix) (*(((char*)(&pix))+0))
#define GREEN(pix) (*(((char*)(&pix))+1))
#define BLUE(pix) (*(((char*)(&pix))+2))
#define ALPHA(pix) (*(((char*)(&pix))+3))

  /* When assigning color channels with the above macros, its  */
  /* important that the color c falls within 0<=c<=255, since  */
  /* we are assigning the value to a 1byte slot. We can use the*/
  /* following macro to do this without overflowing            */
  /* int clr=300;                                              */
  /* long final_color;                                         */
  /* RED(final_color)=SAFECOLOR(clr);                          */
#define SAFECOLOR(c) (((((c)<0)?0:(c))>255)?255:(((c)<0)?0:(c)))

  /* You may find that you need to create temporary images     */
  /* during your calculations. The following functions make it */
  /* easy to create and destroy EF_Images of whatever size.    */
  EF_Image *newEF_Image(short xSize, short ySize);
  void deleteEF_Image(EF_Image *image);

  /* Convenient routine to copy an image.It assumes the two    */
  /* images are the same size.                                 */
  void copyEF_Image(EF_Image *src, EF_Image *dst);
  
  /* Blacks out the pixels in an image                         */
  void blackEF_Image(EF_Image *src);
  
#ifdef __cplusplus
}
#endif

#endif  /* !EF_IMAGE_H */

