//
// EF_Movie.C
//
// $Id: EF_Movie.C,v 1.4 1998/02/08 23:13:00 mmp Exp $
//


#include <dmedia/moviefile.h>
#include <stdio.h>
#include <dmedia/dm_color.h>
#include <dmedia/dm_image.h>
#include <dmedia/dm_params.h>
#include "EF_Movie.h"

void movieCreate(char *filename, MVid *movie, short xsize, short ysize)
{
    DMparams *params;
    MVid track;
  
    if ((dmParamsCreate( &params) !=DM_SUCCESS) ||
	(mvSetMovieDefaults(params, MV_FORMAT_QT)!=DM_SUCCESS) ||
	(mvCreateFile(filename,params,NULL,movie)!=DM_SUCCESS) ||
	(dmSetImageDefaults(params,xsize,ysize,DM_PACKING_APPLE_32))) {
	fprintf(stderr, "error creating SGI movie\n");
    }
	
    dmParamsSetEnum(params,DM_IMAGE_PACKING, DM_PACKING_APPLE_32);
    dmParamsSetEnum(params,DM_IMAGE_ORIENTATION,  DM_TOP_TO_BOTTOM);
    dmParamsSetString(params,DM_IMAGE_COMPRESSION, DM_IMAGE_JPEG);
  
    if (mvAddTrack(*movie, DM_IMAGE, params, NULL, &track)!=DM_SUCCESS) {
	fprintf(stderr, "error creating SGI movie\n");
    }
			 
    dmParamsDestroy(params);	
}

void movieSave(char *filename, EF_Image *image, short frame)
{
    MVid movie;
    DMstatus status;
    MVid track;
 
    status=mvOpenFile(filename, O_RDWR, &movie);
    if (status==DM_FAILURE) {
	movieCreate(filename,&movie,image->xSize,image->ySize);
    }

    status=mvFindTrackByMedium(movie,DM_IMAGE,&track);
    if (status==DM_FAILURE) {
	fprintf(stderr, "error creating SGI movie\n");
	return;
    }
 
    int width=mvGetImageWidth(track);
    int height=mvGetImageHeight(track);
 
    // convert to xbgr
    movieRGBAtoXBGR(image);
 
    // Write frame
    status=mvDeleteFrames(track,frame,1);
    if (status==DM_FAILURE) {
	fprintf(stderr, "error creating SGI movie\n");
	return;
    }

    status=mvInsertFrames(track,frame,1,width*height*4,image->pixels);
    if (status==DM_FAILURE) {
	fprintf(stderr, "error creating SGI movie\n");
	return;
    }

    // convert back to rgba - skip since we don't use it later
    // movieXBGRtoRGBA(image);

    // close movie
    mvClose(movie);
    if (status==DM_FAILURE) {
	fprintf(stderr, "error creating SGI movie\n");
    }
 
}

short movieNumFrames(char *filename)
{
    MVid movie;
    DMstatus status;
    MVid track;
 
    status=mvOpenFile(filename, O_RDONLY, &movie);
    if (status==DM_FAILURE) {
	puts ("couldnt open movie file");
	return 0;
    }

    status=mvFindTrackByMedium(movie,DM_IMAGE,&track);
    if (status==DM_FAILURE) {
	puts ("couldnt open movie track");
	mvClose(movie);
	return 0;
    }

    short tracklen = mvGetTrackLength(track);
 

    mvClose(movie);
    if (status==DM_FAILURE) {
	puts ("couldnt close movie file");
    }

    return tracklen;
 
}

EF_Image *movieLoad(char *filename, short frame)
{
    MVid movie;
    DMstatus status;
    MVid track;
    EF_Image *image = 0;
 
    status=mvOpenFile(filename, O_RDONLY, &movie);
    if (status==DM_FAILURE) {
	puts ("couldnt open movie file");
	return 0;
    }

    //int numTracks = mvGetNumTracks(movie);
    //pr/intf ("Num tracks %d\n",numTracks);
    
    status=mvFindTrackByMedium(movie,DM_IMAGE,&track);
    if (status==DM_FAILURE) {
	puts ("couldnt open movie track");
	return 0;
    }
 
    int width=mvGetImageWidth(track);
    int height=mvGetImageHeight(track);
 
    //  printf ("%d %d \n",width, height);
    
    image = newEF_Image(width, height);
 
    status=mvReadFrames(track,frame,1,width*height*4,image->pixels);
    if (status==DM_FAILURE) {
	puts ("couldnt read frame");
	deleteEF_Image(image);
	return 0;
    }
    mvClose(movie);
    if (status==DM_FAILURE) {
	puts ("couldnt close movie file");
    }
    return image;
}

void movieXBGRtoRGBA(EF_Image *image)
{
    char *outbuf=NULL;
    DMcolorconverter convert;
    DMparams *srcP, *destP;
  
    short xsize=image->xSize;
    short ysize=image->ySize;
   
    outbuf = (char *)malloc(xsize*ysize*4);
   
    dmColorCreate(&convert);
     
    dmParamsCreate(&srcP);
    dmParamsSetInt(srcP, DM_IMAGE_WIDTH, xsize);
    dmParamsSetInt(srcP, DM_IMAGE_HEIGHT, ysize);
    dmParamsSetEnum(srcP, DM_IMAGE_PACKING,
		    DM_IMAGE_PACKING_XBGR );
    dmParamsSetEnum(srcP, DM_IMAGE_ORIENTATION,
		    DM_IMAGE_TOP_TO_BOTTOM);
    dmColorSetSrcParams(convert,srcP);

    dmParamsCreate(&destP);
    dmParamsSetInt(destP, DM_IMAGE_WIDTH, xsize);
    dmParamsSetInt(destP, DM_IMAGE_HEIGHT, ysize);
    dmParamsSetEnum(destP, DM_IMAGE_PACKING,
		    DM_IMAGE_PACKING_RGBA);
    dmParamsSetEnum(destP, DM_IMAGE_ORIENTATION, 
		    DM_IMAGE_BOTTOM_TO_TOP);
    dmColorSetDstParams(convert,destP );
      
    dmColorConvert(convert, image->pixels, outbuf);

    dmParamsDestroy(srcP);
    dmParamsDestroy(destP);
  
    dmColorDestroy(convert);

    free(image->pixels);
    image->pixels=(long *) outbuf;
}

void movieRGBAtoXBGR(EF_Image *image)
{
    char *outbuf=NULL;
    DMcolorconverter convert;
    DMparams *srcP, *destP;
  
    short xsize=image->xSize;
    short ysize=image->ySize;
   
    outbuf = (char *)malloc(xsize*ysize*4);
   
    dmColorCreate(&convert);
     
    dmParamsCreate(&srcP);
    dmParamsSetInt(srcP, DM_IMAGE_WIDTH, xsize);
    dmParamsSetInt(srcP, DM_IMAGE_HEIGHT, ysize);
    dmParamsSetEnum(srcP, DM_IMAGE_PACKING,
		    DM_IMAGE_PACKING_RGBA );
    dmParamsSetEnum(srcP, DM_IMAGE_ORIENTATION,
		    DM_IMAGE_TOP_TO_BOTTOM);
    dmColorSetSrcParams(convert,srcP);

    dmParamsCreate(&destP);
    dmParamsSetInt(destP, DM_IMAGE_WIDTH, xsize);
    dmParamsSetInt(destP, DM_IMAGE_HEIGHT, ysize);
    dmParamsSetEnum(destP, DM_IMAGE_PACKING,
		    DM_IMAGE_PACKING_XBGR);
    dmParamsSetEnum(destP, DM_IMAGE_ORIENTATION, 
		    DM_IMAGE_BOTTOM_TO_TOP);
    dmColorSetDstParams(convert,destP );
      
    dmColorConvert(convert, image->pixels, outbuf);

    dmParamsDestroy(srcP);
    dmParamsDestroy(destP);

    dmColorDestroy(convert);

    free(image->pixels);
    image->pixels=(long *) outbuf;
}
